﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StorageContracts;
using LawFirmContracts.ViewModels;
using LawFirmDataModels.Models;
using LawFirmFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;
        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return source.Shops.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return source.Shops
                .Select(x => x.GetViewModel)
                .Where(x => x.Name.Contains(model.Name ?? string.Empty))
                .ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(shop => shop.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            source.Shops.Remove(shop);
            source.SaveShops();
            return shop.GetViewModel;
        }

        public bool SellDocument(IDocumentModel model, int count)
        {
            var document = source.Documents.FirstOrDefault(x => x.Id == model.Id);

            if (document == null)
            {
                return false;
            }

            var countStore = count;

            var shopDocuments = source.Shops.SelectMany(shop => shop.ShopDocuments.Where(doc => doc.Value.Item1.Id == document.Id));

            foreach (var doc in shopDocuments)
            {
                count -= doc.Value.Item2;

                if (count <= 0)
                {
                    break;
                }
            }

            if (count > 0)
            {
                return false;
            }

            count = countStore;

            foreach (var shop in source.Shops)
            {
                var documents = shop.ShopDocuments;

                foreach (var doc in documents.Where(x => x.Value.Item1.Id == document.Id))
                {
                    var min = Math.Min(doc.Value.Item2, count);
                    documents[doc.Value.Item1.Id] = (doc.Value.Item1, doc.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    Name = shop.Name,
                    Adress = shop.Adress,
                    OpeningDate = shop.OpeningDate,
                    MaxCountDocuments = shop.MaxCountDocuments,
                    ShopDocuments = documents
                });

                source.SaveShops();

                if (count <= 0) break;
            }

            return count <= 0;
        }
    }
}
