﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.ViewModels;
using LawFirmDataModels.Models;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace LawFirmFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }

        public string Name { get; private set; } = string.Empty;

        public string Adress { get; private set; } = string.Empty;

        public int MaxCountDocuments { get; private set; }

        public DateTime OpeningDate { get; private set; }

        public Dictionary<int, int> Documents { get; private set; } = new();

        private Dictionary<int, (IDocumentModel, int)>? _shopDocuments = null;

        public Dictionary<int, (IDocumentModel, int)> ShopDocuments
        {
            get
            {
                if (_shopDocuments == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopDocuments = Documents.ToDictionary(
                        x => x.Key,
                        y => ((source.Documents.FirstOrDefault(z => z.Id == y.Key) as IDocumentModel)!, y.Value)
                    );
                }
                return _shopDocuments;
            }
        }

        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                Name = model.Name,
                Adress = model.Adress,
                MaxCountDocuments = model.MaxCountDocuments,
                OpeningDate = model.OpeningDate,
                Documents = model.ShopDocuments.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                Name = element.Element("Name")!.Value,
                Adress = element.Element("Address")!.Value,
                MaxCountDocuments = Convert.ToInt32(element.Element("MaxCountDocuments")!.Value),
                OpeningDate = Convert.ToDateTime(element.Element("OpeningDate")!.Value),
                Documents= element.Element("ShopDocuments")!.Elements("ShopDocument").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value)
                )
            };
        }


        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Name = model.Name;
            Adress = model.Adress;
            OpeningDate = model.OpeningDate;
            MaxCountDocuments = model.MaxCountDocuments;
            if (model.ShopDocuments.Count > 0)
            {
                Documents = model.ShopDocuments.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopDocuments = null;
            }
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            Name = Name,
            Adress = Adress,
            OpeningDate = OpeningDate,
            ShopDocuments = ShopDocuments,
            MaxCountDocuments = MaxCountDocuments
        };

        public XElement GetXElement => new(
            "Shop",
             new XAttribute("Id", Id),
             new XElement("Name", Name),
             new XElement("Address", Adress),
             new XElement("MaxCountDocuments", MaxCountDocuments),
             new XElement("OpeningDate", OpeningDate.ToString()),
             new XElement("ShopDocuments", Documents.Select (x =>
             new XElement("ShopDocument",
             new XElement("Key", x.Key),
             new XElement("Value", x.Value)))
            .ToArray()));
    }
}

