﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StorageContracts;
using LawFirmContracts.ViewModels;
using LawFirmListImplements.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmListImplements.Implements
{
    public class BlankStorage : IBlankStorage
    {
        private readonly DataListSingleton _source;

        public BlankStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public BlankViewModel? GetElement(BlankSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BlankName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var blank in _source.Blanks)
            {
                if ((!string.IsNullOrEmpty(model.BlankName) && blank.BlankName == model.BlankName) || (model.Id.HasValue && blank.Id == model.Id))
                {
                    return blank.GetViewModel;
                }
            }
            return null;
        }

        public List<BlankViewModel> GetFilteredList(BlankSearchModel model)
        {
            var result = new List<BlankViewModel>();
            if (string.IsNullOrEmpty(model.BlankName))
            {
                return result;
            }
            foreach (var blank in _source.Blanks)
            {
                if (blank.BlankName.Contains(model.BlankName))
                {
                    result.Add(blank.GetViewModel);
                }
            }
            return result;
        }

        public List<BlankViewModel> GetFullList()
        {
            var result = new List<BlankViewModel>();
            foreach (var blank in _source.Blanks)
            {
                result.Add(blank.GetViewModel);
            }
            return result;
        }

        public BlankViewModel? Insert(BlankBindingModel model)
        {
            model.Id = 1;
            foreach (var blank in _source.Blanks)
            {
                if (model.Id <= blank.Id)
                {
                    model.Id = blank.Id + 1;
                }
            }
            var newBlank = Blank.Create(model);
            if (newBlank == null)
            {
                return null;
            }
            _source.Blanks.Add(newBlank);
            return newBlank.GetViewModel;
        }

        public BlankViewModel? Update(BlankBindingModel model)
        {
            foreach (var blank in _source.Blanks)
            {
                if (blank.Id == model.Id)
                {
                    blank.Update(model);
                    return blank.GetViewModel;
                }
            }
            return null;
        }
        public BlankViewModel? Delete(BlankBindingModel model)
        {
            for (int i = 0; i < _source.Blanks.Count; ++i)
            {
                if (_source.Blanks[i].Id == model.Id)
                {
                    var element = _source.Blanks[i];
                    _source.Blanks.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
